@@ -11,9 +11,10 @@ use core::ops::{Deref, DerefMut};
 
 use canonical::{Canon, CanonError, Id, Sink, Source};
 
-#[cfg(feature = "persistance")]
-use crate::persist::{PersistError, Persistance};
+#[cfg(feature = "persistence")]
+use crate::persist::{PersistError, Persistence};
 
+use crate::generic::GenericTree;
 use crate::{Annotation, Compound};
 
 #[derive(Debug, Clone)]
@@ -27,11 +28,25 @@ enum LinkInner<C, A> {
 }
 
 #[derive(Clone)]
-/// TODO
+/// The Link struct is an annotated merkle link to a compound type
+///
+/// The link takes care of lazily evaluating the annotation of the inner type,
+/// and to load it from memory or backend when needed.
 pub struct Link<C, A> {
     inner: RefCell<LinkInner<C, A>>,
 }
 
+impl<C, A> Default for Link<C, A>
+where
+    C: Default,
+{
+    fn default() -> Self {
+        Link {
+            inner: RefCell::new(LinkInner::C(Rc::new(C::default()))),
+        }
+    }
+}
+
 impl<C: core::fmt::Debug, A: core::fmt::Debug> core::fmt::Debug for Link<C, A> {
     fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
         write!(f, "{:?}", self.inner)
@@ -97,8 +112,8 @@ where
     where
         C::Leaf: Canon,
     {
-        // assure compound is loaded
-        let _ = self.compound()?;
+        // assure inner value is loaded
+        let _ = self.inner()?;
 
         let inner = self.inner.into_inner();
         match inner {
@@ -113,7 +128,7 @@ where
         }
     }
 
-    /// Computes the Id of the
+    /// Computes the Id of the link
     pub fn id(&self) -> Id
     where
         C::Leaf: Canon,
@@ -130,10 +145,16 @@ where
         }
     }
 
+    /// See doc for `inner`
+    #[deprecated(since = "0.10.0", note = "Please use `inner` instead")]
+    pub fn compound(&self) -> Result<LinkCompound<C, A>, CanonError> {
+        self.inner()
+    }
+
     /// Gets a reference to the inner compound of the link'
     ///
     /// Can fail when trying to fetch data over i/o
-    pub fn compound(&self) -> Result<LinkCompound<C, A>, CanonError> {
+    pub fn inner(&self) -> Result<LinkCompound<C, A>, CanonError> {
         let borrow = self.inner.borrow();
 
         match *borrow {
@@ -141,53 +162,107 @@ where
             LinkInner::C(_) | LinkInner::Ca(_, _) | LinkInner::Ica(_, _, _) => {
                 return Ok(LinkCompound(borrow))
             }
-            LinkInner::Ia(_, _) => {
-                #[cfg(feature = "persistance")]
-                {
-                    // re-borrow mutable
-                    drop(borrow);
-                    let mut borrow = self.inner.borrow_mut();
-                    if let LinkInner::Ia(id, anno) =
-                        mem::replace(&mut *borrow, LinkInner::Placeholder)
-                    {
-                        match Persistance::get(&id) {
-                            Ok(generic) => {
-                                let compound = C::from_generic(&generic)?;
-                                *borrow =
-                                    LinkInner::Ica(id, Rc::new(compound), anno);
+            LinkInner::Ia(id, _) => {
+                // First we check if the value is available to be reified
+                // directly
+                match id.reify::<GenericTree>() {
+                    Ok(generic) => {
+                        // re-borrow mutable
+                        drop(borrow);
+                        let mut borrow = self.inner.borrow_mut();
+                        if let LinkInner::Ia(id, anno) =
+                            mem::replace(&mut *borrow, LinkInner::Placeholder)
+                        {
+                            let value = C::from_generic(&generic)?;
+                            *borrow = LinkInner::Ica(id, Rc::new(value), anno);
 
-                                // re-borrow immutable
-                                drop(borrow);
-                                let borrow = self.inner.borrow();
+                            // re-borrow immutable
+                            drop(borrow);
+                            let borrow = self.inner.borrow();
 
-                                Ok(LinkCompound(borrow))
-                            }
-                            Err(PersistError::Canon(e)) => Err(e),
-                            Err(PersistError::Io(_)) => {
-                                Err(CanonError::NotFound)
+                            Ok(LinkCompound(borrow))
+                        } else {
+                            unreachable!(
+                                "Guaranteed to match the same as above"
+                            )
+                        }
+                    }
+                    Err(CanonError::NotFound) => {
+                        // Value was not able to be reified, if we're using
+                        // persistance we look in the backend, otherwise we
+                        // return an `Err(NotFound)`
+
+                        #[cfg(feature = "persistence")]
+                        {
+                            // re-borrow mutable
+                            drop(borrow);
+                            let mut borrow = self.inner.borrow_mut();
+                            if let LinkInner::Ia(id, anno) = mem::replace(
+                                &mut *borrow,
+                                LinkInner::Placeholder,
+                            ) {
+                                match Persistence::get(&id) {
+                                    Ok(generic) => {
+                                        let compound =
+                                            C::from_generic(&generic)?;
+                                        *borrow = LinkInner::Ica(
+                                            id,
+                                            Rc::new(compound),
+                                            anno,
+                                        );
+
+                                        // re-borrow immutable
+                                        drop(borrow);
+                                        let borrow = self.inner.borrow();
+
+                                        Ok(LinkCompound(borrow))
+                                    }
+                                    Err(PersistError::Canon(e)) => Err(e),
+                                    err
+                                    @
+                                    Err(
+                                        PersistError::Io(_)
+                                        | PersistError::Other(_),
+                                    ) => {
+                                        // TODO: log errors to the backend
+                                        Err(CanonError::NotFound)
+                                    }
+                                }
+                            } else {
+                                unreachable!(
+                                    "Guaranteed to match the same as above"
+                                )
                             }
                         }
-                    } else {
-                        unreachable!()
+                        #[cfg(not(feature = "persistence"))]
+                        Err(CanonError::NotFound)
                     }
+                    Err(e) => Err(e),
                 }
-                #[cfg(not(feature = "persistance"))]
-                Err(CanonError::NotFound)
             }
         }
     }
 
+    /// See doc for `inner_mut`
+    #[deprecated(since = "0.10.0", note = "Please use `inner` instead")]
+    pub fn compound_mut(&mut self) -> Result<LinkCompoundMut<C, A>, CanonError>
+    where
+        C: Canon,
+    {
+        self.inner_mut()
+    }
+
     /// Returns a Mutable reference to the underlying compound node
     ///
     /// Drops cached annotations and ids
     ///
     /// Can fail when trying to fetch data over i/o
-    pub fn compound_mut(&mut self) -> Result<LinkCompoundMut<C, A>, CanonError>
+    pub fn inner_mut(&mut self) -> Result<LinkCompoundMut<C, A>, CanonError>
     where
         C: Canon,
     {
-        // assure compound is loaded
-        let _ = self.compound()?;
+        // assure inner value is loaded
+        let _ = self.inner()?;
 
         let mut borrow: RefMut<LinkInner<C, A>> = self.inner.borrow_mut();
 
